import os
from cStringIO import StringIO
from django.core.files.uploadedfile import InMemoryUploadedFile


def truncate_file(f):
    """
    Clear uploaded file and allow write to it.
    Only for not too big files!!!
    Also can clear simple opened file.
    Examples:
        truncate_file(request.FILES['file'])

        with open('/tmp/file', 'rb+') as f:
            truncate_file(f)
    """
    if isinstance(f, InMemoryUploadedFile):
        f.file = StringIO()
    else:
        f.seek(0)
        f.truncate(0)


def make_dirs_for_file_path(file_path, mode=0o775):
    """
    Make dirs for file file_path, if these dirs are not exist.
    """
    dirname = os.path.dirname(file_path)
    if not os.path.exists(dirname):
        os.makedirs(dirname, mode=mode)



from django import template
from ..constants import DJU_EMAIL_BLOCK_MARKERS


register = template.Library()


class EmailBlockNode(template.Node):
    def __init__(self, nodelist, markers):
        self.nodelist = nodelist
        self.markers = markers

    def render(self, context):
        return u'{1}{0}{2}'.format(self.nodelist.render(context), *self.markers)


@register.tag
def dju_email_subject(parser, token):
    nodelist = parser.parse(('enddju_email_subject', 'end_dju_email_subject'))
    parser.delete_first_token()
    return EmailBlockNode(nodelist, DJU_EMAIL_BLOCK_MARKERS['subject'])


@register.tag
def dju_email_plain_body(parser, token):
    nodelist = parser.parse(('enddju_email_plain_body', 'end_dju_email_plain_body'))
    parser.delete_first_token()
    return EmailBlockNode(nodelist, DJU_EMAIL_BLOCK_MARKERS['plain_body'])


@register.tag
def dju_email_html_body(parser, token):
    nodelist = parser.parse(('enddju_email_html_body', 'end_dju_email_html_body'))
    parser.delete_first_token()
    return EmailBlockNode(nodelist, DJU_EMAIL_BLOCK_MARKERS['html_body'])



from django.core.cache import cache
from dju_common.management import LoggingBaseCommand


class Command(LoggingBaseCommand):
    help = 'Clear cache.'

    def handle(self, *args, **options):
        self.log('Clear cache...')
        cache.clear()
        self.log('Done')



DJU_EMAIL_BLOCK_MARKERS = {
    'subject': ('<!--__DJU_EMAIL_SUBJECT_BEGIN__-->', '<!--__DJU_EMAIL_SUBJECT_END__-->'),
    'plain_body': ('<!--__DJU_EMAIL_PLAIN_BODY_BEGIN__-->', '<!--__DJU_EMAIL_PLAIN_BODY_END__-->'),
    'html_body': ('<!--__DJU_EMAIL_HTML_BODY_BEGIN__-->', '<!--__DJU_EMAIL_HTML_BODY_END__-->'),
}



from django.shortcuts import render
from django.views.decorators.csrf import requires_csrf_token


@requires_csrf_token
def http400(request):
    return render(request, 'http_error/400.html', {'request_path': request.get_full_path()}, status=400)


@requires_csrf_token
def http403(request):
    return render(request, 'http_error/403.html', {'request_path': request.get_full_path()}, status=403)


@requires_csrf_token
def http404(request):
    return render(request, 'http_error/404.html', {'request_path': request.get_full_path()}, status=404)


@requires_csrf_token
def http500(request):
    return render(request, 'http_error/500.html', {'request_path': request.get_full_path()}, status=500)



import copy
import re
from django.core.mail import EmailMultiAlternatives
from django.template import loader
from django.template.loader import render_to_string
from django.conf import settings
from django.utils import translation, timezone
from django.utils.module_loading import import_string
from django.utils.translation import ugettext as _
from .constants import DJU_EMAIL_BLOCK_MARKERS
from . import settings as dju_settings


def attach_html_wrapper(content, title=None, head=None):
    return render_to_string('dju_common/mail/html_wrapper.html', {'content': content, 'title': title, 'head': head})


def send_mail(subject, body, to, from_email=None, reply_to=None,
              fail_silently=False, attach_alternative=None, attaches=None):
    headers = {}
    if dju_settings.DJU_EMAIL_RETURN_PATH:
        headers['Return-Path'] = dju_settings.DJU_EMAIL_RETURN_PATH
    if settings.EMAIL_SUBJECT_PREFIX:
        subject = _(settings.EMAIL_SUBJECT_PREFIX) + subject
    if isinstance(to, basestring):
        to = (to,)
    if reply_to is None:
        reply_to = dju_settings.DJU_EMAIL_REPLY_TO or None
    if isinstance(reply_to, basestring):
        reply_to = [reply_to]
    mail = EmailMultiAlternatives(subject=subject, body=body, from_email=from_email, to=to, headers=headers,
                                  reply_to=reply_to)
    if attach_alternative:
        for attach in attach_alternative:
            mail.attach_alternative(*attach)
    if attaches:
        for attach in attaches:
            mail.attach(*attach)
    return mail.send(fail_silently)


class RenderMailSender(object):
    TEMPLATE_EXT = '.html'
    BLOCKS_RE = {
        'subject': re.compile(r'{}(.*?){}'.format(*DJU_EMAIL_BLOCK_MARKERS['subject']), re.U | re.S),
        'plain_body': re.compile(r'{}(.*?){}'.format(*DJU_EMAIL_BLOCK_MARKERS['plain_body']), re.U | re.S),
        'html_body': re.compile(r'{}(.*?){}'.format(*DJU_EMAIL_BLOCK_MARKERS['html_body']), re.U | re.S),
    }

    class TemplateEmailTagNotFound(Exception):
        pass

    def __init__(self, tpl_fn, context=None, request=None, from_email=None, reply_to=None,
                 fail_silently=False, attach_alternative=None, attaches=None, lang=None, tz=None):
        self._tpl_fn = None
        self._context = None
        self._request = None
        self._from_email = None
        self._reply_to = None
        self._fail_silently = None
        self._attach_alternative = None
        self._attaches = None
        self._lang = None
        self._tz = None
        self._allow_configs = ('tpl_fn', 'context', 'request', 'from_email', 'reply_to', 'fail_silently',
                               'attach_alternative', 'attaches', 'lang', 'tz')
        self.configure(tpl_fn=tpl_fn, context=context, request=request, from_email=from_email, reply_to=reply_to,
                       fail_silently=fail_silently, attach_alternative=attach_alternative, attaches=attaches,
                       lang=lang, tz=tz)
        self._tpl = None
        self._lang_old = None
        self._tz_old = None
        self._render_cache = None
        self._context_cache = None

    def configure(self, **kwargs):
        for k, v in kwargs.iteritems():
            if k not in self._allow_configs:
                raise AttributeError('Unknown attr "%s"' % k)
            setattr(self, '_' + k, v)
            if k in ('tpl_fn', 'lang', 'tz'):
                self._tpl = None
                self._render_cache = None
            elif k in ('request', 'context'):
                self._context_cache = None
                self._render_cache = None

    def _get_tpl(self):
        if self._tpl is None:
            self._tpl = loader.select_template((
                '{fn}_{lang}{ext}'.format(fn=self._tpl_fn, lang=translation.get_language(), ext=self.TEMPLATE_EXT),
                '{fn}{ext}'.format(fn=self._tpl_fn, ext=self.TEMPLATE_EXT),
                self._tpl_fn,
            ))
        return self._tpl

    def _get_context(self):
        if self._context_cache is None:
            self._context_cache = {} if self._context is None else copy.copy(self._context)
            if not self._request:
                if dju_settings.DJU_EMAIL_DEFAULT_CONTEXT:
                    self._context_cache.update(import_string(dju_settings.DJU_EMAIL_DEFAULT_CONTEXT)())
        return self._context_cache

    @classmethod
    def _plain_normalize(cls, plain):
        r = re.compile(r'^` (.+)')
        plain = '\n'.join(r.sub(r'  \1', line.strip()) for line in plain.replace('\r', '').split('\n'))
        plain = re.sub(r'\n{3,}', '\n\n', plain)
        return plain

    def _set_locale(self):
        self._lang_old = translation.get_language()
        self._tz_old = timezone.get_current_timezone_name()
        if self._lang and self._lang != self._lang_old:
            translation.activate(self._lang)
        else:
            self._lang_old = None
        if self._tz and self._tz != self._tz_old:
            timezone.activate(self._tz)
        else:
            self._tz_old = None

    def _back_locale(self):
        if self._lang_old:
            translation.activate(self._lang_old)
        if self._tz_old:
            timezone.activate(self._tz_old)

    def send(self, to):
        if not isinstance(to, (list, tuple)):
            to = (to,)
        self._set_locale()
        try:
            if self._render_cache is None:
                tpl = self._get_tpl()
                r = tpl.render(context=self._get_context(), request=self._request)
                for block, markers in DJU_EMAIL_BLOCK_MARKERS.iteritems():
                    for marker in markers:
                        if marker not in r:
                            raise self.TemplateEmailTagNotFound('Require tag not used: dju_email_{}.'.format(block))
                contents = {
                    'subject': self.BLOCKS_RE['subject'].search(r).group(1).strip(),
                    'plain_body': self._plain_normalize(self.BLOCKS_RE['plain_body'].search(r).group(1).strip()),
                    'html_body': self.BLOCKS_RE['html_body'].search(r).group(1).strip(),
                }
                self._render_cache = contents
                del r
            else:
                contents = self._render_cache
            attach_alternative = tuple(self._attach_alternative) if self._attach_alternative else ()
            if contents['html_body']:
                attach_alternative += ((contents['html_body'], 'text/html'),)
            return send_mail(contents['subject'], contents['plain_body'], to,
                             from_email=self._from_email, reply_to=self._reply_to, fail_silently=self._fail_silently,
                             attach_alternative=attach_alternative, attaches=self._attaches)
        finally:
            self._back_locale()



from types import NoneType
from django.core.cache import cache as default_cache
from django.core.cache.backends.base import DEFAULT_TIMEOUT


CACHE_TAG_KEY = '_tcache_tag:%s'


def cache_set(key, value, timeout=DEFAULT_TIMEOUT, version=None, tags=None, cache=None, **kwargs):
    assert isinstance(tags, (NoneType, list, basestring, set))
    if cache is None:
        cache = default_cache
    r = cache.set(key, value, timeout=timeout, version=version, **kwargs)
    if not tags:
        return r
    if isinstance(tags, basestring):
        tags = [tags]
    tag_keys = [CACHE_TAG_KEY % tag for tag in tags]
    if cache.__class__.__name__ == 'RedisCache':
        from django_redis.exceptions import ConnectionInterrupted
        try:
            redis_client = cache.client.get_client()
            for tag_key in tag_keys:
                redis_client.sadd(tag_key, key)
        except ConnectionInterrupted:
            pass  # todo add logging
    else:
        for tag_key in tag_keys:
            keys = cache.get(tag_key) or set()
            keys.add(key)
            cache.set(tag_key, keys, timeout=None)
    return r


def cache_invalidate_by_tags(tags, cache=None):
    """
    Clear cache by tags.
    """
    if isinstance(tags, basestring):
        tags = [tags]
    tag_keys = [CACHE_TAG_KEY % tag for tag in tags if tag]
    if not tag_keys:
        raise ValueError('Attr tags invalid')
    if cache is None:
        cache = default_cache
    tag_keys_for_delete = []
    if cache.__class__.__name__ == 'RedisCache':
        from django_redis.exceptions import ConnectionInterrupted
        try:
            redis_client = cache.client.get_client()
            for tag_key in tag_keys:
                keys = redis_client.smembers(tag_key)
                if keys:
                    cache.delete_many(keys)
                    tag_keys_for_delete.append(tag_key)
        except ConnectionInterrupted:
            pass  # todo add logging
    else:
        for tag_key in tag_keys:
            keys = cache.get(tag_key)
            if keys:
                cache.delete_many(keys)
                tag_keys_for_delete.append(tag_key)
    if tag_keys_for_delete:
        cache.delete_many(tag_keys_for_delete)


# todo add decorators for caching result from function or class method



from django.http.response import HttpResponseBase
from django.views.generic import TemplateView
from dju_common.http import send_json


class TemplateViewWithAjax(TemplateView):
    """
    CBV with AJAX support.

    Example:
        class MyView(TemplateViewWithAjax):
            ...
            def _ajax_test_json(self):
                return {'result': 'test'}

            def _ajax_test_html(self):
                return HttpResponse('<h1>Test</h1>')

        frontend:
            $.ajax(
              method: 'POST'
              data:
                csrfmiddlewaretoken: $.cookie('csrf-token')
                action: 'test_json'
            ).done((data) ->
              alert data.result
            )

            $.ajax(
              method: 'POST'
              data:
                csrfmiddlewaretoken: $.cookie('csrf-token')
                action: 'test_html'
            ).done((data) ->
              if typeof data is 'string'
                $('body').append(data)
              else
                console.log data.errors
            )
    """
    unknown_ajax_action_result = {'ok': 0, 'errors': ['Attr action invalid.']}  # if attr action is invalid

    def dispatch(self, request, *args, **kwargs):
        if request.is_ajax():
            resp = self._ajax()
            if not isinstance(resp, HttpResponseBase):
                resp = send_json(resp)
            return resp
        return super(TemplateViewWithAjax, self).dispatch(request, *args, **kwargs)

    def _ajax(self):
        method = '_ajax_' + self.request.POST.get('action', '')
        if hasattr(self, method) and callable(getattr(self, method)):
            return getattr(self, method)()
        return self.unknown_ajax_action_result



import mimetypes
import os
import urllib
import urlparse
import simplejson
from wsgiref.util import FileWrapper
from django.contrib.sites.models import Site
from django.http import HttpResponse, HttpResponsePermanentRedirect, HttpResponseRedirect
from django.shortcuts import resolve_url
from . import settings as dju_settings


def encode_url_query_params(params):
    assert isinstance(params, dict)
    result = {}
    for k, v in params.iteritems():
        result[k] = unicode(v).encode('utf-8')
    return result


def resolve_url_ext(to, params_=None, anchor_=None, args=None, kwargs=None):
    """
    Advanced resolve_url which can includes GET-parameters and anchor.
    """
    url = resolve_url(to, *(args or ()), **(kwargs or {}))
    if params_:
        url += '?' + urllib.urlencode(encode_url_query_params(params_))
    if anchor_:
        url += '#' + anchor_
    return url


def send_file(file_path, content_type=None, as_attachment=False, filename=None, encoding=None):
    if content_type is None:
        content_type, encoding = mimetypes.guess_type(file_path)
    response = HttpResponse(FileWrapper(file(file_path, 'rb')), content_type=content_type)
    if as_attachment:
        if filename is None:
            filename = os.path.basename(file_path)
        response['Content-Disposition'] = "attachment; filename*=UTF-8''{}".format(
            urllib.quote(filename.encode('utf-8'))
        )
    response['Content-Length'] = os.stat(file_path).st_size
    if encoding is not None:
        response['Content-Encoding'] = encoding
    return response


def send_json(data, content_type='application/json', status=200, json_dumps_kwargs=None):
    if json_dumps_kwargs is None:
        json_dumps_kwargs = {}
    return HttpResponse(
        simplejson.dumps(data, **json_dumps_kwargs),
        content_type=content_type,
        status=status
    )


def send_jsonp(data, f='jsonpCallback', status=200, json_dumps_kwargs=None):
    if json_dumps_kwargs is None:
        json_dumps_kwargs = {}
    return HttpResponse(
        '{}({});'.format(f, simplejson.dumps(data, **json_dumps_kwargs)),
        content_type='application/javascript',
        status=status
    )


def redirect_ext(to, params_=None, anchor_=None, permanent_=False, args=None, kwargs=None):
    """
    Advanced redirect which can includes GET-parameters and anchor.
    """
    if permanent_:
        redirect_class = HttpResponsePermanentRedirect
    else:
        redirect_class = HttpResponseRedirect
    return redirect_class(resolve_url_ext(to, params_, anchor_, args, kwargs))


def change_url_query_params(url, params):
    """
    Add GET-parameters to URL. If parameters are exist then they will be changed.
        url - URL, string value
        params - dict {'GET-parameter name': 'value'}
    """
    url_parts = list(urlparse.urlparse(url))
    query = dict(urlparse.parse_qsl(url_parts[4]))
    query.update(params)
    url_parts[4] = urllib.urlencode(encode_url_query_params(query))
    return urlparse.urlunparse(url_parts)


def add_response_headers(h):
    """
    Add HTTP-headers to response.
    Example:
        @add_response_headers({'Refresh': '10', 'X-Powered-By': 'Django'})
        def view(request):
            ....
    """
    def headers_wrapper(fun):
        def wrapped_function(*args, **kwargs):
            response = fun(*args, **kwargs)
            for k, v in h.iteritems():
                response[k] = v
            return response
        return wrapped_function
    return headers_wrapper


def full_url(path=None, secure=None, request=None):
    if secure is None:
        secure = dju_settings.USE_HTTPS
    site = Site.objects.get_current(request)
    return '{}://{}{}'.format((secure and 'https' or 'http'), site.domain, path or '')



import os
from django.conf import settings
from django.core.management import BaseCommand, call_command
from django.apps.registry import apps as project_apps


class Command(BaseCommand):
    help = 'Compile locale .po files for applications of project.'

    def _get_paths_of_apps(self):
        result = []
        for app in project_apps.app_configs.keys() + ['.']:
            if app == '.':
                path = settings.BASE_DIR
            else:
                path = os.path.abspath(project_apps.app_configs[app].path)
            assert os.path.isdir(path)
            if not path.startswith(settings.BASE_DIR):
                continue
            if not os.path.isdir(os.path.join(path, 'locale').replace('\\', '/')):
                continue
            result.append(path)
        if not result:
            self.stdout.write('Apps not found.')
            exit(1)
        return list(set(result))

    def handle(self, *args, **options):
        for path in self._get_paths_of_apps():
            os.chdir(path)
            self.stdout.write('chdir {}'.format(path))
            call_command('compilemessages')



import sys
import os
import subprocess
import errno
from copy import copy
from django.core.management import BaseCommand, CommandError
from django.conf import settings
from django.apps.registry import apps as project_apps


class Command(BaseCommand):
    help = 'Make locale .po files for applications of project.'
    SPLIT_TEMPLATE_APP_DIR = True
    SPLIT_STATIC_APP_DIR = True

    def add_arguments(self, parser):
        super(Command, self).add_arguments(parser)
        parser.add_argument('--application', '-a', action='append', dest='apps', default=[], metavar='APPNAME',
                            help='The application name. If not set then using all project app. '
                                 '(Project locale named ".")')
        parser.add_argument('--locale', '-l', action='append', dest='locale', default=None,
                            help='Creates or updates the message files for the given locale(s) (e.g. pt_BR). '
                                 'Can be used multiple times, accepts a comma-separated list of locale names.')
        parser.add_argument('--domain', '-d', action='append', dest='domain', default=None,
                            help='The domain of the message files (default: ["django", "djangojs"]).')
        parser.add_argument('--extension', '-e', action='append', dest='extensions', default=None,
                            help='The file extension(s) to examine (default: "html,txt,rml", or "js" if the domain is '
                                 '"djangojs"). Separate multiple extensions with commas, or use -e multiple times.')
        parser.add_argument('--no-update-header', action='store_true', dest='no_update_header', default=False,
                            help="Don't update POT-Creation-Date in headers.")

    def _get_paths_of_apps(self, apps):
        result = []
        for app in apps or (project_apps.app_configs.keys() + ['.']):
            if app == '.':
                path = settings.BASE_DIR
            else:
                path = os.path.abspath(project_apps.app_configs[app].path)
            assert os.path.isdir(path)
            if not path.startswith(settings.BASE_DIR):
                if apps:
                    raise CommandError('App "%s" is not in project path.' % app)
                continue
            if not os.path.isdir(os.path.join(path, 'locale').replace('\\', '/')):
                self.stdout.write('App "%s" ignore because locale path not found.' % app)
                continue
            result.append(path)
        if not result:
            exit('App list is empty.')
        return list(set(result))

    @staticmethod
    def _get_paths_of_apps_with_locale():
        result = []
        for app in project_apps.app_configs.values():
            app_path = os.path.abspath(app.path)
            assert os.path.isdir(app_path)
            if not app_path.startswith(settings.BASE_DIR):
                continue
            if not os.path.isdir(os.path.join(app_path, 'locale').replace('\\', '/')):
                continue
            result.append(app_path.rstrip('/'))
        return result

    @staticmethod
    def _make_locale_dirs(path):
        for lang_code, lang_name in settings.LANGUAGES:
            dn = os.path.join(path, 'locale', lang_code, 'LC_MESSAGES').replace('\\', '/')
            try:
                os.makedirs(dn, 0775)
            except OSError, e:
                if not (e.errno == errno.EEXIST and os.path.isdir(dn)):
                    raise e

    def handle(self, *args, **options):
        paths = self._get_paths_of_apps(options['apps'])
        command_base = [sys.executable, os.path.join(settings.BASE_DIR, 'manage.py').replace('\\', '/'),
                        'dju_custom_django_makemessages']
        if options['locale']:
            for lang in options['locale']:
                command_base.extend(['-l', lang])
        else:
            command_base.append('-a')
        if options['no_update_header']:
            command_base.append('--no-update-header')
        if not options['domain']:
            options['domain'] = ['django', 'djangojs']
        for path in paths:
            os.chdir(path)
            rel_path = lambda p: os.path.relpath(p, path)
            for domain in options['domain']:
                command = copy(command_base)
                extensions = copy(options['extensions'])
                command.extend(['-d', domain])
                if not extensions:
                    if domain == 'django':
                        extensions = ['html', 'txt', 'rml', 'py']
                    elif domain == 'djangojs':
                        extensions = ['js']
                if extensions:
                    command.extend(['-e', ','.join(extensions)])
                if path == settings.BASE_DIR:
                    for app_path in self._get_paths_of_apps_with_locale():
                        command.extend(['-i', '{}/*'.format(rel_path(app_path))])
                    if self.SPLIT_TEMPLATE_APP_DIR:  # exclude templates/appname
                        for app_path in self._get_paths_of_apps_with_locale():
                            command.extend(['-i', '{}/*'.format(
                                rel_path(os.path.join(settings.BASE_DIR, 'templates', os.path.basename(app_path)))
                            )])
                    if self.SPLIT_STATIC_APP_DIR:  # exclude static/appname
                        for app_path in self._get_paths_of_apps_with_locale():
                            command.extend(['-i', '{}/*'.format(
                                rel_path(os.path.join(settings.BASE_DIR, 'static', os.path.basename(app_path)))
                            )])
                else:
                    locale_dir = os.path.join(path, 'locale')
                    locale_dir_suffix = ':{}'.format(rel_path(locale_dir)) if os.path.isdir(locale_dir) else ''
                    if self.SPLIT_TEMPLATE_APP_DIR:  # include templae/appname
                        command.extend([
                            '--add-source-dir',
                            rel_path(
                                os.path.join(settings.BASE_DIR, 'templates', os.path.basename(path))
                            ) + locale_dir_suffix,
                        ])
                    if self.SPLIT_STATIC_APP_DIR:  # include static/appname
                        command.extend([
                            '--add-source-dir',
                            rel_path(
                                os.path.join(settings.BASE_DIR, 'static', os.path.basename(path))
                            ) + locale_dir_suffix,
                        ])
                self.stdout.write('[%s] %s' % (path, ' '.join(command)))
                self._make_locale_dirs(path)
                subprocess.call(command)



import urllib
import hashlib
from . import settings as dju_settings


def get_ava_url(email, size=None):
    if not size:
        size = dju_settings.DJU_GRAVATAR_DEFAULT_SIZE
    return '{}://www.gravatar.com/avatar/{}?{}'.format(
        (dju_settings.DJU_GRAVATAR_SECURE and 'https' or 'http'),
        hashlib.md5(email.lower()).hexdigest(),
        urllib.urlencode({
            'd': dju_settings.DJU_GRAVATAR_DEFAULT_IMAGE,
            's': str(size),
            'r': dju_settings.DJU_GRAVATAR_RATING
        })
    )


def get_profile_url(email):
    return '{}://gravatar.com/{}'.format(
        (dju_settings.DJU_GRAVATAR_SECURE and 'https' or 'http'),
        hashlib.md5(email.lower()).hexdigest()
    )



from django.core.paginator import Paginator, PageNotAnInteger, EmptyPage


def paginate(request, objects, count=10, param_name='page', param_perpage=None, **kwargs):
    default_last = kwargs.pop('default_last', False)
    if objects is None:
        objects = []
    if param_perpage and param_perpage in request.GET and request.GET[param_perpage].isdigit():
        count = int(request.GET[param_perpage])
    paginator = Paginator(objects, count, **kwargs)
    default_page = paginator.num_pages if default_last else 1
    try:
        page = paginator.page(request.GET.get(param_name, default_page))
    except PageNotAnInteger:
        page = paginator.page(default_page)
    except EmptyPage:
        page = paginator.page(paginator.num_pages)
    page.param_name = param_name
    return page



import copy
import datetime
import simplejson
from types import NoneType
from django.db import models
from django.forms.widgets import Textarea
from django.utils.timezone import is_aware
from django.utils.translation import ugettext_lazy as _
from django.forms.fields import CharField
from django.forms.utils import ValidationError


JSON_INVALID = ValidationError(_('Enter valid JSON.'))


class JSONFormField(CharField):
    def __init__(self, *args, **kwargs):
        self.load_kwargs = kwargs.pop('load_kwargs', {})
        self.dump_kwargs = kwargs.pop('dump_kwargs', {})
        super(JSONFormField, self).__init__(*args, **kwargs)

    def to_python(self, value):
        if not value and not self.required:
            return None
        if isinstance(value, basestring):
            try:
                value = simplejson.loads(value, **self.load_kwargs)
            except ValueError:
                raise JSON_INVALID
        return value

    def prepare_value(self, value):
        if isinstance(value, basestring):
            value = simplejson.loads(value, **self.load_kwargs)
        kwargs = self.dump_kwargs.copy()
        kwargs['sort_keys'] = True
        if isinstance(self.widget, Textarea):
            kwargs['indent'] = 4
            kwargs['separators'] = (',', ': ')
            kwargs['ensure_ascii'] = False
        return simplejson.dumps(value, **kwargs)


class JSONEncoder(simplejson.JSONEncoder):
    """
    JSONEncoder subclass that knows how to encode date, time, datetime.
    """
    def default(self, o):
        if isinstance(o, datetime.datetime):
            r = o.isoformat()
            if o.microsecond:
                r = r[:23] + r[26:]
            if r.endswith('+00:00'):
                r = r[:-6] + 'Z'
            return r
        elif isinstance(o, datetime.date):
            return o.isoformat()
        elif isinstance(o, datetime.time):
            if is_aware(o):
                raise ValueError(_("JSON can't represent timezone-aware times."))
            r = o.isoformat()
            if o.microsecond:
                r = r[:12]
            return r
        else:
            return super(JSONEncoder, self).default(o)


class JSONFieldBase(models.Field):
    DEFAULT_SEPARATORS = (',', ':')

    def __init__(self, *args, **kwargs):
        if 'default' not in kwargs:
            kwargs['default'] = None
        elif not isinstance(kwargs['default'], (list, dict, NoneType)) and not callable(kwargs['default']):
            raise AssertionError('Default can be None, list or dict.')
        if 'blank' not in kwargs:
            kwargs['blank'] = True
        use_decimal = kwargs.pop('use_decimal', False)
        self.dump_kwargs = {'cls': JSONEncoder,
                            'separators': self.DEFAULT_SEPARATORS,
                            'use_decimal': use_decimal}
        self.dump_kwargs.update(kwargs.pop('dump_kwargs', {}))
        self.load_kwargs = {'use_decimal': use_decimal}
        self.load_kwargs.update(kwargs.pop('load_kwargs', {}))
        super(JSONFieldBase, self).__init__(*args, **kwargs)

    def to_python(self, value):
        if isinstance(value, basestring):
            if value == '':
                return None
            try:
                return simplejson.loads(value, **self.load_kwargs)
            except ValueError:
                raise JSON_INVALID
        return value

    def from_db_value(self, value, expression, connection, context):
        if value in (None, ''):
            return None
        return simplejson.loads(value, **self.load_kwargs)

    def get_prep_value(self, value):
        if value is None and self.null:
            return None
        return simplejson.dumps(value, **self.dump_kwargs)

    def get_default(self):
        return self.default() if callable(self.default) else copy.deepcopy(self.default)

    def formfield(self, **kwargs):
        if 'form_class' not in kwargs:
            kwargs['form_class'] = JSONFormField
        field = super(JSONFieldBase, self).formfield(**kwargs)
        if not field.help_text:
            field.help_text = _('JSON data')
        if isinstance(field, JSONFormField):
            field.load_kwargs = self.load_kwargs
            field.dump_kwargs = self.dump_kwargs
        return field

    def deconstruct(self):
        name, path, args, kwargs = super(JSONFieldBase, self).deconstruct()
        kwargs['dump_kwargs'] = self.dump_kwargs
        kwargs['load_kwargs'] = self.load_kwargs
        return name, path, args, kwargs


class JSONField(JSONFieldBase, models.TextField):
    pass


class JSONCharField(JSONFieldBase, models.CharField):
    pass



from __future__ import absolute_import
from django.utils.text import capfirst
from modeltranslation.settings import AVAILABLE_LANGUAGES, DEFAULT_LANGUAGE


def mt_fields(fields, nomaster=False, onlydefaultlang=False):
    """
    Returns list of fields for multilanguage fields of model.
    Examples:
        print(mt_fields('name', 'desc'))
        ['name', 'name_en', 'name_uk', 'desc', 'desc_en', 'desc_uk']

        MyModel.objects.only(*mt_fields('name', 'desc', 'content'))

    If nomaster then master field will not be append.
    F.e.: ['name_en', 'name_uk'] -- without master 'name'.

    If onlydefaultlang then wiil be search only default language:
    F.e.: ['name', 'name_en'] -- without additional 'name_uk'.

    If nomaster and onlydefaultlang then will be use both rulses.
    F.e.: ['name_en'] -- without master 'name' and additional 'name_uk'.
    """
    assert isinstance(fields, (list, tuple))
    fl = []
    for field in fields:
        if not nomaster:
            fl.append(field)
        if onlydefaultlang:
            fl.append('{}_{}'.format(field, DEFAULT_LANGUAGE))
        else:
            for lang in AVAILABLE_LANGUAGES:
                fl.append('{}_{}'.format(field, lang))
    return fl


def mt_modelform_field_restore_original_label(field, make_capfirst=False):
    """
    If label of field has suffix with language, you can fix it, for example:

    class MyModelForm(models.ModelForm):
        class Meta:
            model = MyModel
            _mt_fields = mt_fields(('title', 'description'), nomaster=True)
            fields = _mt_fields

        def __init__(self, *args, **kwargs):
            super(ItemCategoryEditForm, self).__init__(*args, **kwargs)

            for field_name in self.Meta._mt_fields:
                mt_modelform_field_restore_original_label(self.fields[field_name], make_capfirst=True)
    """
    field.label = field.label._proxy____args[0]._proxy____args[0]
    if make_capfirst:
        field.label = capfirst(field.label)


def mt_modelform_register_clean_method(form_self, field_name, func, nomaster=False):
    """
    You can add clean_<field_name> for each translated field. For example:

    class MyModelForm(models.ModelForm):
        class Meta:
            model = MyModel
            _mt_fields = mt_fields(('title', 'description'), nomaster=True)
            fields = _mt_fields

        def __init__(self, *args, **kwargs):
            super(ItemCategoryEditForm, self).__init__(*args, **kwargs)

            mt_modelform_register_clean_method(self, 'title', self.mt_clean_title, nomaster=True)

        def mt_clean_title(self, field_name, lang):
            value = self.cleaned_data[field_name]
            # validation here
            return value
    """
    args_list = [('{}_{}'.format(field_name, lang), lang) for lang in AVAILABLE_LANGUAGES]
    if not nomaster:
        args_list.append((field_name, None))

    def _get_mt_clean_method(args):
        def _mt_clean_method():
            return func(*args)
        return _mt_clean_method

    for item_args in args_list:
        method_name = 'clean_{}'.format(item_args[0])
        setattr(form_self, method_name, _get_mt_clean_method(item_args))



from __future__ import absolute_import
from django import template
from django.core.cache.utils import make_template_fragment_key
from django.core.cache import cache
from dju_common.tcache import cache_set


register = template.Library()


class CacheNode(template.Node):
    def __init__(self, nodelist, expire_time_var, fragment_name, vary_on, tags):
        self.nodelist = nodelist
        self.expire_time_var = expire_time_var
        self.fragment_name = fragment_name
        self.vary_on = vary_on
        self.tags = tags

    def get_tags(self, context):
        if self.tags:
            tags = self.tags.resolve(context)
            if isinstance(tags, basestring):
                tags = [tag.strip() for tag in tags.split(',')]
            elif not isinstance(tags, (list, tuple, set)):
                raise template.TemplateSyntaxError('"tcache" tag variable "tags" invalid')
            return filter(None, tags) or None

    def render(self, context):
        try:
            expire_time = self.expire_time_var.resolve(context)
            expire_time = int(expire_time)
        except (ValueError, TypeError):
            raise template.TemplateSyntaxError('"tcache" tag got a non-integer timeout value: %r' % expire_time)
        vary_on = [var.resolve(context) for var in self.vary_on]
        cache_key = make_template_fragment_key(self.fragment_name, vary_on)
        value = cache.get(cache_key)
        if value is None:
            value = self.nodelist.render(context)
            cache_set(cache_key, value, timeout=expire_time, tags=self.get_tags(context))
        return value


@register.tag
def tcache(parser, token):
    """
    This will cache the contents of a template fragment for a given amount
    of time with support tags.

    Usage::
        {% tcache [expire_time] [fragment_name] [tags='tag1,tag2'] %}
            .. some expensive processing ..
        {% endtcache %}

    This tag also supports varying by a list of arguments:
        {% tcache [expire_time] [fragment_name] [var1] [var2] .. [tags=tags] %}
            .. some expensive processing ..
        {% endtcache %}

    Each unique set of arguments will result in a unique cache entry.
    """
    nodelist = parser.parse(('endtcache',))
    parser.delete_first_token()
    tokens = token.split_contents()
    if len(tokens) < 3:
        raise template.TemplateSyntaxError("'%r' tag requires at least 2 arguments." % tokens[0])
    tags = None
    if len(tokens) > 3 and 'tags=' in tokens[-1]:
        tags = parser.compile_filter(tokens[-1][5:])
        del tokens[-1]
    return CacheNode(nodelist,
        parser.compile_filter(tokens[1]),
        tokens[2],  # fragment_name can't be a variable.
        [parser.compile_filter(token) for token in tokens[3:]],
        tags
    )



import simplejson
import datetime
import pytz
from django.utils.dateparse import parse_datetime, parse_date, parse_time
from django.utils.timezone import is_aware


# todo cover tests


class JSONEncoder(simplejson.JSONEncoder):
    @staticmethod
    def custom_obj(custom_type, data):
        return {'@_type_': custom_type, '@_data_': data}

    def default(self, o):
        # See "Date Time String Format" in the ECMA-262 specification.
        if isinstance(o, datetime.datetime):
            if is_aware(o):
                o = o.astimezone(pytz.utc)
            r = o.isoformat()
            if o.microsecond:
                r = r[:23] + r[26:]
            return self.custom_obj('datetime', r)
        elif isinstance(o, datetime.date):
            return self.custom_obj('date', o.isoformat())
        elif isinstance(o, datetime.time):
            r = o.isoformat()
            if o.microsecond:
                r = r[:12]
            return self.custom_obj('time', r)
        else:
            return super(JSONEncoder, self).default(o)


class JSONSerializer(object):
    @classmethod
    def object_hook(cls, d):
        s = d.get('@_data_')
        if s:
            dt = d.get('@_type_')
            if dt == 'datetime':
                return parse_datetime(s)
            elif dt == 'date':
                return parse_date(s)
            elif dt == 'time':
                return parse_time(s)
        return d

    @classmethod
    def dumps(cls, obj):
        return simplejson.dumps(obj, cls=JSONEncoder, separators=(',', ':'))

    @classmethod
    def loads(cls, data):
        return simplejson.loads(data, use_decimal=True, object_hook=cls.object_hook)



import os
import threading
import imp
import time
import datetime
from django.conf import settings


_ts_start = time.mktime(datetime.datetime.now().timetuple())


def mts_now():
    global _ts_start
    dt = datetime.datetime.now()
    return (time.mktime(dt.timetuple()) - _ts_start) * 1000 + dt.microsecond / 1000


mts_start = mts_now()


class CyclicTaskBase(object):
    run_wait = 1000      # wait before first execution, ms
    run_interval = None  # execution interval, ms

    def __init__(self):
        self.last_run = mts_start + self.run_wait
        self.is_running = False
        self.thread = None

    def task(self):
        raise NotImplementedError('subclasses of CyclicTaskBase must provide a task() method')

    def run(self):
        self.is_running = True
        self.thread = threading.Thread(target=self.run_task)
        self.last_run = mts_now()
        self.thread.start()

    def run_task(self):
        self.task()
        self.is_running = False

    @classmethod
    def register(cls):
        CyclicTaskManager.register(cls)


class CyclicTaskManager(object):
    CYCLE_INTERVAL = 0.01
    _registered_tasks = []

    @classmethod
    def register(cls, task):
        assert issubclass(task, CyclicTaskBase)
        task = task()
        assert isinstance(task.run_interval, (int, long))
        assert task not in cls._registered_tasks
        cls._registered_tasks.append(task)

    @classmethod
    def start(cls):
        while True:
            now = mts_now()
            for t in cls._registered_tasks:
                if not t.is_running and (t.last_run + t.run_interval) < now:
                    t.run()
            time.sleep(cls.CYCLE_INTERVAL)

    @classmethod
    def count_tasks(cls):
        return len(cls._registered_tasks)


_manager_thread = None


def init(pid_fn=None):
    global _manager_thread
    if _manager_thread is not None:
        return
    for app in settings.INSTALLED_APPS:
        try:
            imp.find_module('cyclictask', __import__(app, fromlist=[app.split('.')[-1]]).__path__)
        except (AttributeError, ImportError):
            continue
        __import__('%s.cyclictask' % app)
    _manager_thread = False
    if CyclicTaskManager.count_tasks():
        _manager_thread = threading.Thread(target=CyclicTaskManager.start, name='cyclictask')
        _manager_thread.setDaemon(True)
        _manager_thread.start()
        created_pid = False
        try:
            if pid_fn:
                if os.path.exists(pid_fn):
                    try:
                        pid = int(open(pid_fn).read())
                    except (OSError, TypeError):
                        pass
                    else:
                        try:
                            if os.name == 'nt':
                                os.popen('TASKKILL /PID {pid} /F'.format(pid=pid))
                            else:
                                os.kill(pid, 0)
                        except OSError:
                            pass
                with open(pid_fn, 'w') as f:
                    f.write(str(os.getpid()))
                    created_pid = True
            try:
                while _manager_thread.is_alive():
                    _manager_thread.join(1)
            except (KeyboardInterrupt, SystemExit):
                pass
        finally:
            if created_pid and os.path.exists(pid_fn):
                try:
                    os.remove(pid_fn)
                except OSError:
                    pass



from __future__ import absolute_import
from django import template
from dju_common import gravatar


register = template.Library()


@register.filter
def gravatar_ava_url(email, size=None):
    return gravatar.get_ava_url(email, size=size)


@register.filter
def gravatar_profile_url(value):
    return gravatar.get_profile_url(value)



import os
from django.conf import settings


# ------------
# COMMON SETTINGS
# ------------
LOG_DIR = getattr(settings, 'LOG_DIR', os.path.join(settings.BASE_DIR, 'logs').replace('\\', '/'))
USE_HTTPS = getattr(settings, 'USE_HTTPS', False)


# ------------
# SENDING EMAIL
# ------------
DJU_EMAIL_RETURN_PATH = getattr(settings, 'DJU_EMAIL_RETURN_PATH', None)
DJU_EMAIL_REPLY_TO = getattr(settings, 'DJU_EMAIL_REPLY_TO', None)  # list of emails
DJU_EMAIL_DOMAIN_BLACK_LIST = getattr(settings, 'DJU_EMAIL_DOMAIN_BLACK_LIST', (
    'mailinator.com', '10minutemail.com', 'spambog.com', 'tempinbox.com', 'mailmetrash.com',
    'tempemail.net', 'yopmail.com', 'sharklasers.com', 'guerrillamailblock.com', 'guerrillamail.com',
    'guerrillamail.net', 'guerrillamail.biz', 'guerrillamail.org', 'guerrillamail.de', 'spam4.me', 'spam.su',
    'inboxed.im', 'inboxed.pw', 'gomail.in', 'tokem.co', 'nomail.pw', 'yanet.me', 'powered.name', 'shut.ws',
    'vipmail.pw', 'powered.im', 'linuxmail.so', 'secmail.pw', 'shut.name', 'freemail.ms', 'mailforspam.com',
    'uroid.com', 'rmqkr.net',
))
DJU_EMAIL_DEFAULT_CONTEXT = getattr(settings, 'DJU_EMAIL_DEFAULT_CONTEXT',
                                    'dju_common.context_processors.email_default')


# ------------
# EMAIL DEBUGGING
# ------------
DJU_EMAIL_DEBUG_IN_CONSOLE = getattr(settings, 'DJU_EMAIL_DEBUG_IN_CONSOLE', True)
DJU_EMAIL_DEBUG_IN_FILES = getattr(settings, 'DJU_EMAIL_DEBUG_IN_FILES', True)
DJU_EMAIL_DEBUG_PATH = getattr(settings, 'DJU_EMAIL_DEBUG_PATH',
                               os.path.join(LOG_DIR, 'debug_email').replace('\\', '/'))


# ------------
# GRAVATAR
# ------------
DJU_GRAVATAR_DEFAULT_SIZE = getattr(settings, 'DJU_GRAVATAR_DEFAULT_SIZE', 50)
# 404, mm, identicon, monsterid, wavatar, retro, blank, http://mysite.com/default.jpg
DJU_GRAVATAR_DEFAULT_IMAGE = getattr(settings, 'DJU_GRAVATAR_DEFAULT_IMAGE', 'mm')
DJU_GRAVATAR_RATING = getattr(settings, 'DJU_GRAVATAR_RATING', 'g')  # g, pg, r, x
DJU_GRAVATAR_SECURE = getattr(settings, 'DJU_GRAVATAR_SECURE', USE_HTTPS)



import os
from cStringIO import StringIO
from django.core.management.commands import makemessages


class Command(makemessages.Command):
    def add_arguments(self, parser):
        super(Command, self).add_arguments(parser)
        parser.add_argument('--add-source-dir', action='append', dest='additional_source_dirs',
                            default=[], metavar='DIR',
                            help='Additional source directory. For example: "/path/to/src:/path/to/locale"')
        parser.add_argument('--no-update-header', action='store_true', dest='no_update_header', default=False,
                            help="Don't update POT-Creation-Date in headers.")

    def handle(self, *args, **options):
        self.additional_source_dirs = options['additional_source_dirs']
        self.no_update_header = options['no_update_header']
        super(Command, self).handle(*args, **options)

    def find_files(self, root):
        files = super(Command, self).find_files(root)
        for d in self.additional_source_dirs:
            if ':' in d:
                src_dir, locale_dir = d.split(':', 1)
            else:
                src_dir, locale_dir = d, None
            add_files = super(Command, self).find_files(os.path.abspath(src_dir))
            for add_file in add_files:
                add_file.dirpath = os.path.relpath(add_file.dirpath, os.getcwd())
                if locale_dir:
                    add_file.locale_dir = locale_dir
            files.extend(add_files)
        return sorted(files)

    def write_po_file(self, potfile, locale):
        pofile_fn = os.path.join(os.path.dirname(potfile), locale, 'LC_MESSAGES', '{}.po'.format(self.domain))
        old_meta = {}  # {'line start': 'full line'}
        if self.no_update_header and os.path.isfile(pofile_fn):
            with open(pofile_fn, 'r+') as f:
                for line in f:
                    if line.startswith('msgid ') and not line.startswith('msgid ""'):
                        break
                    for start_line in ('"POT-Creation-Date:',):
                        if line.startswith(start_line):
                            old_meta[start_line] = line
                            break

        super(Command, self).write_po_file(potfile, locale)

        if old_meta:
            with open(pofile_fn, 'r+') as f:
                tmp_f = StringIO()
                for line in f:
                    for start_line, replace_line in old_meta.items():
                        if line.startswith(start_line):
                            line = replace_line
                            old_meta.pop(start_line)
                            break
                    tmp_f.write(line)
                f.seek(0)
                f.write(tmp_f.getvalue())



from __future__ import absolute_import
import simplejson
from django import template
from django.template.loader import get_template
from django.http import QueryDict
from django.utils.safestring import mark_safe, mark_for_escaping
from django.utils.encoding import force_unicode
from dju_common.http import full_url
from dju_common.tools import long_number_readable


register = template.Library()


@register.filter
def of_key(var, key):
    try:
        return var[key]
    except (KeyError, TypeError, IndexError):
        return ''


@register.filter
def of_intkey(var, key):
    try:
        return var[int(key)]
    except (KeyError, TypeError, IndexError):
        return ''


@register.filter
def of_strkey(var, key):
    try:
        return var[str(key)]
    except (KeyError, TypeError):
        return ''


@register.filter
def is_in(val, items):
    """
    {% if tag|is_in:'div|p|span' %}is block tag{% endif %}
    {% if tag|is_in:tags_list %}is block tag{% endif %}
    """
    if isinstance(items, basestring):
        items = items.split('|')
    return val in items


class CaptureasNode(template.Node):
    def __init__(self, nodelist, args):
        self.nodelist = nodelist
        self.varname = args[1]
        self.assign_and_print = len(args) > 2 and bool(args[2])

    def render(self, context):
        output = self.nodelist.render(context)
        context[self.varname] = output
        return(output if self.assign_and_print else '')


@register.tag
def captureas(parser, arg):
    """
    example: {% captureas myvar 1 %}content...{% endcaptureas %} - {{ myvar }}
    result: content... - content...

    example: {% captureas myvar %}content...{% endcaptureas %} - {{ myvar }}
    result: - content...
    """
    args = arg.contents.split()
    if not 2 <= len(args) <= 3:
        raise template.TemplateSyntaxError('"captureas" node requires a variable name and/or assign only')
    nodelist = parser.parse(('endcaptureas',))
    parser.delete_first_token()
    return CaptureasNode(nodelist, args)


class StripNode(template.Node):
    def __init__(self, nodelist, args):
        if len(args) > 2:
            raise template.TemplateSyntaxError("'%s' tag can have one parameter" % args[0])
        self.nodelist = nodelist
        self.separator = args[1][1:-1] if len(args) == 2 else ' '
        self.separator = self.separator.replace('{\\n}', '\n')

    def render(self, context):
        output = self.nodelist.render(context)
        return self.separator.join(filter(bool, [line.strip() for line in output.splitlines()]))


@register.tag
def strip(parser, arg):
    """
    example:
        {% strip '<br>' %}
            content...
            content...
        {% endstrip %}
    result:
        content...<br>content...
    arg special symbols: {\n} = \n
    """
    nodelist = parser.parse(('endstrip',))
    parser.delete_first_token()
    return StripNode(nodelist, arg.split_contents())


class IncludeNode(template.Node):
    def __init__(self, template_name):
        self.template_name = template_name

    def render(self, context):
        try:
            try:
                included_template = context.template.engine.get_template(self.template_name).render(context)
            except AttributeError:
                included_template = get_template(self.template_name).render(context)
        except template.TemplateDoesNotExist:
            included_template = ''
        return included_template


@register.tag
def include_silently(parser, token):
    """
    Include template if it exists.
    If it doesn't exist then will not raise any exception.
    {% include_silently "mytemplate.html" %}
    """
    try:
        tag_name, template_name = token.split_contents()
    except ValueError:
        raise template.TemplateSyntaxError('%r tag requires a single argument' % token.contents.split()[0])
    return IncludeNode(template_name[1:-1])


class RecurseNode(template.Node):
    def __init__(self, template_nodes, items_var, children_attr):
        self.template_nodes = template_nodes
        self.items_var = items_var
        self.children_attr = children_attr

    def _render(self, context, item, level=1):
        t = []
        context.push()
        children = ()
        if hasattr(item, self.children_attr):
            children = getattr(item, self.children_attr, ())
            if callable(children):
                children = children()
        elif hasattr(item, 'get'):
            children = item.get(self.children_attr, ())
            if callable(children):
                children = children()
        for child in children or ():
            t.append(self._render(context, child, level=level + 1))
        context['item'] = item
        context['subitems'] = mark_safe(''.join(t))
        context['recurse_level'] = level
        rendered = self.template_nodes.render(context)
        context.pop()
        return rendered

    def render(self, context):
        return ''.join(self._render(context, item) for item in self.items_var.resolve(context))


@register.tag
def recurse(parser, token):
    """
    Iterate recurse data structure.
    <ul>
        {% recurse items %}
            <li>
                {{ item.name }}
                {% if item.children %}
                    <ul>
                        {{ subitems }}
                    </ul>
                {% endif %}
            </li>
        {% endrecurse %}
    </ul>
    If subelements found in other key/attribute/method then need set its name (default is 'children'):
        {% recurse items 'subitems' %}
    Also available depth level in variable {{ recurse_level }} (starting of 1)
    """
    params = token.contents.split()
    if not 2 <= len(params) <= 3:
        raise template.TemplateSyntaxError('%s parameters error' % params[0])
    template_nodes = parser.parse(('endrecurse',))
    parser.delete_first_token()
    return RecurseNode(template_nodes,
                       template.Variable(params[1]),
                       (params[2][1:-1] if len(params) == 3 else 'children'))


@register.filter
def humanize_long_number(value):
    """
    Convert big integer (>=999) to readable form.
    """
    return long_number_readable(value)

    
@register.filter
def int_subtract(a, b):
    """
    Subtract the arg of the value.
    """
    try:
        return int(a) - int(b)
    except (ValueError, TypeError):
        return ''


@register.simple_tag
def assign(value):
    """
    Setting value to a variable.
    {% assign obj.get_full_name as obj_full_name %}
    """
    return value


@register.simple_tag(takes_context=True)
def assign_default(context, var_name, default):
    """
    Sets default value to variable if one does not exist.
    """
    if var_name not in context:
        context[var_name] = default
    return ''


@register.simple_tag
def assign_format_str(string, *args, **kwargs):
    """
    Format string and save it to variable.
    {% assign_format_str 'contacts_{lang}.html' lang=LANGUAGE_CODE as tpl_name %}
    {% include tpl_name %}
    """
    return string.format(*args, **kwargs)


@register.filter
def tojson(value):
    """
    Convert value to json-object.
        var data = '{{ data|tojson|safe }}';
        <div data-data="{{ data|tojson }}"></div>
    """
    return simplejson.dumps(value)


@register.inclusion_tag(('paginator.html', 'dju_common/tags/paginator.html'), takes_context=True)
def paginator(context, page, leading=8, out=3, adjacent=3, sep='...'):
    """
    Render paginator.
    <<_1_2 3 4 5 6 7 8 9 ... 55 56 57 >>  # leading=9 (1..9)  out=3 (55..57)
    << 1 2 3 ... 23 24 25 26_27_28 29 30 31 ... 55 56 57 >>  # adjacent = 4 (23..26, 28..31)
    """
    leading_pages, out_left_pages, out_right_pages, pages = [], [], [], []
    num_pages = page.paginator.num_pages
    is_paginated = num_pages > 1
    if is_paginated:
        if num_pages < leading:
            leading_pages = [x for x in range(1, leading) if x <= num_pages]
        elif page.number <= leading - 1:
            leading_pages = [x for x in range(1, leading + 1)]
            out_right_pages = [x + num_pages for x in range(-out + 1, 1) if x + num_pages > leading]
        elif page.number > num_pages - leading + 1:
            leading_pages = [x for x in range(num_pages - leading + 1, num_pages + 1) if x <= num_pages]
            out_left_pages = [x for x in range(1, out + 1) if num_pages - x >= leading]
        else:
            leading_pages = [x for x in range(page.number - adjacent, page.number + adjacent + 1)]
            out_right_pages = [x + num_pages for x in range(-out + 1, 1)]
            out_left_pages = [x for x in range(1, out + 1)]
    if out_left_pages:
        pages.extend(out_left_pages)
        pages.append(sep)
    pages.extend(leading_pages)
    if out_right_pages:
        pages.append(sep)
        pages.extend(out_right_pages)
    return {
        'page': page,
        'pages': pages,
        'is_paginated': is_paginated,
        'separator': sep,
        'request': context.get('request'),
        'param_name': page.param_name if hasattr(page, 'param_name') else 'page',
    }


@register.filter
def str_equal(a, b):
    return force_unicode(a) == force_unicode(b)


@register.filter
def html_encode(value):
    """
    Encode text to HTML special symbols.
    Example:
        <a href="{{ 'mailto:my@example.com'|html_encode }}">{{ 'my@example.com'|html_encode }}</a>
    Result:
        <a href="&#109;&#97;&#105;&#108;&#116;&#111;&#58;&#109;&#121;&#64;...">&#109;&#121;&#64;&#101;&#120;...</a>
    """
    return mark_safe(''.join('&#%d;' % ord(c) for c in value))


def _url_get_amps(token=None):
    l, r, amp = '', '', '&'
    if token and len(token) > 1:
        token = token.strip()
        if token.startswith('&'):
            l = amp
        if token.endswith('&'):
            r = amp
        token = token.strip('&')
    return token, l, r


def _url_getvars(context, token_=None, type_=None):
    assert type_ in (None, 'with', 'without')
    request_get = getattr(context.get('request'), 'GET', QueryDict(''))
    token, l, r = _url_get_amps(token_)
    gv = ''
    if type_:
        w, wo = type_ == 'with', type_ == 'without'
        if w or wo:
            lst = [p.strip() for p in token.split(',') if p.strip()]
            if w and not lst:
                pass
            elif wo and not lst:
                type_ = None
            else:
                params = QueryDict('', mutable=True)
                if w:
                    q = lambda x: x
                else:
                    q = lambda x: not x
                for key in request_get:
                    if q(key in lst):
                        params.setlist(key, request_get.getlist(key))
                gv = params.urlencode()
    if not type_:
        gv = request_get.urlencode()
    if gv:
        gv = l + gv + r
    return gv


@register.simple_tag(takes_context=True)
def url_getvars(context, token=None):
    return mark_for_escaping(_url_getvars(context, token))


@register.simple_tag(takes_context=True)
def url_getvars_with(context, token):
    return mark_for_escaping(_url_getvars(context, token, 'with'))


@register.simple_tag(takes_context=True)
def url_getvars_without(context, token):
    return mark_for_escaping(_url_getvars(context, token, 'without'))


@register.simple_tag(takes_context=True)
def full_url_prefix(context, secure=None):
    request = context.get('request', None)
    return full_url(secure=secure, request=request)


@register.filter
def add_full_url_prefix(value, arg=None):
    return mark_safe(full_url(path=value, secure=arg))



# coding=utf-8
import os
import re
import time
import pytz
import datetime
from django.utils.dateparse import parse_datetime
from django.utils import timezone
from django.conf import settings
from django.utils.translation import ugettext_lazy as _
from . import settings as dju_settings


def natural_sorted(iterable, key=None, cmp_func=None, reverse=False):
    reg = re.compile(r'([0-9]+)')
    convert = lambda text: int(text) if text.isdigit() else text.lower()
    if key is None:
        key = lambda t: t
    alphanum_key = lambda val: [convert(c) for c in reg.split(key(val))]
    return sorted(iterable, key=alphanum_key, cmp=cmp_func, reverse=reverse)


def int2base36(n):
    """
    Convert int base10 to base36.
    Back convert: int('<base36>', 36)
    """
    assert isinstance(n, (int, long))
    c = '0123456789abcdefghijklmnopqrstuvwxyz'
    if n < 0:
        return '-' + int2base36(-n)
    elif n < 36:
        return c[n]
    b36 = ''
    while n != 0:
        n, i = divmod(n, 36)
        b36 = c[i] + b36
    return b36


def datetime_to_dtstr(dt=None):
    """
    Comvert datetime to short text.
    If datetime has timezone then it will be convert to UTC0.
    """
    if dt is None:
        dt = datetime.datetime.utcnow()
    elif timezone.is_aware(dt):
        dt = dt.astimezone(tz=pytz.UTC)
    return int2base36(int(time.mktime(dt.timetuple()) * 1e3 + dt.microsecond / 1e3))


def dtstr_to_datetime(dtstr, to_tz=None, fail_silently=True):
    """
    Convert result from datetime_to_dtstr to datetime in timezone UTC0.
    """
    try:
        dt = datetime.datetime.utcfromtimestamp(int(dtstr, 36) / 1e3)
        if to_tz:
            dt = timezone.make_aware(dt, timezone=pytz.UTC)
            if to_tz != pytz.UTC:
                dt = dt.astimezone(to_tz)
        return dt
    except ValueError, e:
        if not fail_silently:
            raise e
        return None


def load_object_by_name(obj_name):
    """
    Import object by name (point-path).
    Example:
        MyForm = load_object_by_name('myapp.forms.MyAnyForm')
    """
    parts = obj_name.split('.')
    t = __import__('.'.join(parts[:-1]), fromlist=(parts[-1],))
    return getattr(t, parts[-1])

    
def parse_datetime_aware(s, tz=None):
    """
    Parse text with datetime to aware datetime (with timezone).
    """
    assert settings.USE_TZ
    if isinstance(s, datetime.datetime):
        return s
    d = parse_datetime(s)
    if d is None:
        raise ValueError
    return timezone.make_aware(d, tz or timezone.get_current_timezone())


_long_number_readable_formats = (
    (10 ** 6, 10 ** 3, u'{:.0f}', u'{:.1f}', _(u'k.')),
    (10 ** 9, 10 ** 6, u'{:.0f}', u'{:.2f}', _(u' mln.')),
    (10 ** 12, 10 ** 9, u'{:.0f}', u'{:.3f}', _(u' bln.')),
)


def long_number_readable(value):
    """
    Convert big int (>=999) to readable form.
    1000 => 1k.
    2333 => 2.3k.
    1000000 => 1 mln.
    1258000 => 1.26 mln.
    """
    value = int(value)
    if value < 1000:
        return value
    for m, d, inf, fnf, n in _long_number_readable_formats:
        if value < m:
            return ((inf if value % d == 0 else fnf) + unicode(n)).format(value / float(d))
    return value


def log_to_file(msg, double_br=False, add_time=True, fn=None):
    fn = fn or os.path.join(settings.LOG_DIR, 'log_to_file.log').replace('\\', '/')
    with open(fn, 'a') as f:
        msg += '\n' * (int(bool(double_br)) + 1)
        if add_time:
            msg = u'[{}] {}'.format(datetime.datetime.now().strftime(u'%d.%m.%Y %H:%M:%S'), msg)
        f.write(msg.encode('utf8'))


def log_memory_usage(desc='test'):
    try:
        import psutil
    except ImportError:
        raise ImportError("Can't import psutil. Please, install psutil (pip install psutil).")
    proc = psutil.Process(os.getpid())
    mem = proc.get_memory_info()[0] / float(2 ** 20)
    log_to_file('{}: {} MB'.format(desc, mem), add_time=False,
                fn=os.path.join(dju_settings.LOG_DIR, 'memory_usage.log').replace('\\', '/'))



import traceback
from django.conf import settings
from dju_common.mail import RenderMailSender
from dju_common.management import LoggingBaseCommand


class Command(LoggingBaseCommand):
    help = 'Send the test email.'
    args = '<emails>'

    def handle(self, *args, **options):
        if not args:
            self.log('Please, pass the email list as args: '
                     'manage.py send_test_email mail1@domain.com email2@domain.com ...', add_time=False)
            return
        m = RenderMailSender('dju_common/mail/test.html', lang=settings.LANGUAGE_CODE)
        for email in set(args):
            self.log('Send the test email to {email}... '.format(email=email), ending='')
            try:
                m.send(email)
                self.log('OK', add_time=False)
            except Exception, e:
                self.log('FAIL', add_time=False)
                self.log(traceback.format_exc(), add_time=False)
                raise e
        self.log('Done')



from django import forms


DEFAULT_VISIBLE_WIDGETS = (forms.TextInput, forms.Textarea, forms.Select, forms.FileInput)


def add_css_class_to_fields_widget(form_fields, css_class, widget_types=DEFAULT_VISIBLE_WIDGETS):
    for k in form_fields:
        if isinstance(form_fields[k].widget, widget_types):
            add_css_class_to_field(form_fields[k], css_class)


def add_css_class_to_field(form_field, css_class):
    attrs = form_field.widget.attrs
    if 'class' in attrs:
        if attrs['class']:
            attrs['class'] += ' '
    else:
        attrs['class'] = ''
    attrs['class'] += css_class



import os
from distutils.core import setup
from setuptools import find_packages
import dju_common


setup(
    name='dju-common',
    version=dju_common.__version__,
    description='Django Utils: Common Library',
    long_description=open(os.path.join(os.path.dirname(__file__), 'README.rst')).read(),
    license='MIT',
    author='Igor Melnyk @liminspace',
    author_email='liminspace@gmail.com',
    url='https://github.com/liminspace/dju-common',
    packages=find_packages(exclude=('tests', 'tests.*')),
    include_package_data=True,
    zip_safe=False,  # because include static
    install_requires=[
        'django>=1.8,<1.12',
        'simplejson',
    ],
)



from django.db.models import OneToOneField
from django.db.models.fields.related_descriptors import ReverseOneToOneDescriptor
from django.db.transaction import atomic


class AutoSingleRelatedObjectDescriptor(ReverseOneToOneDescriptor):
    @atomic
    def __get__(self, instance, instance_type=None):
        model = getattr(self.related, 'related_model', self.related.model)
        try:
            return super(AutoSingleRelatedObjectDescriptor, self).__get__(instance, instance_type)
        except model.DoesNotExist:
            model.objects.get_or_create(**{self.related.field.name: instance})
            return super(AutoSingleRelatedObjectDescriptor, self).__get__(instance, instance_type)


class AutoOneToOneField(OneToOneField):
    """
    OneToOneField creates related object on first call if it doesnt exist yet.
    Use it instead of original OneToOne field.
    Example:
        class MyProfile(models.Model):
            user = AutoOneToOneField(User, primary_key=True)
            ...
    """
    def contribute_to_related_class(self, cls, related):
        setattr(cls, related.get_accessor_name(), AutoSingleRelatedObjectDescriptor(related))



# coding=utf-8
import gc
from django.shortcuts import _get_queryset


def get_object_or_None(klass, *args, **kwargs):
    """
    Uses get() to return an object or None if the object does not exist.
    klass may be a Model, Manager, or QuerySet object.
    All other passed arguments and keyword arguments are used in the get() query.
    """
    queryset = _get_queryset(klass)
    try:
        return queryset.get(*args, **kwargs)
    except queryset.model.DoesNotExist:
        return None


def chunked_qs(qs, order_by='pk', chunksize=1000, yield_values=True):
    qs = qs.order_by(order_by)
    if order_by.startswith('-'):
        fn = 'lt'
        ord_field = order_by[1:]
    else:
        fn = 'gt'
        ord_field = order_by
    last_ordered_val = None
    empty = False
    while not empty:
        empty = True
        chunk_qs = qs
        if last_ordered_val is not None:
            chunk_qs = chunk_qs.filter(**{'{}__{}'.format(ord_field, fn): last_ordered_val})
        chunk_qs = chunk_qs[:chunksize]
        if yield_values:
            row = None
            for row in chunk_qs:
                yield row
            if row is not None:
                last_ordered_val = getattr(row, ord_field)
                empty = False
        else:
            rows = tuple(chunk_qs)
            yield rows
            if rows:
                last_ordered_val = getattr(rows[-1], ord_field)
                empty = False
        gc.collect()


def each_fields(for_fields, fields):
    """
    select_related(
        'field__related_field1__text1', 'field__related_field1__text2',
        'field__related_field2__text1', 'field__related_field2__text2',
    )
    select_related(*each_fields(['field__related_field1', 'field__related_field2'], ['text1', 'text2']))
    each_fields('field__related_field1', ['text1', 'text2'])
    """
    if isinstance(for_fields, basestring):
        for_fields = (for_fields,)
    r = set()
    for ff in for_fields:
        for f in fields:
            r.add(ff + '__' + f)
    return list(r)



from django.core.exceptions import ValidationError
from django.utils.translation import ugettext as _
from . import settings as dju_settings


def validate_email_domain(email):
    """ Validates email domain by blacklist. """
    try:
        domain = email.split('@', 1)[1].lower().strip()
    except IndexError:
        return
    if domain in dju_settings.DJU_EMAIL_DOMAIN_BLACK_LIST:
        raise ValidationError(_(u'Email with domain "%(domain)s" is disallowed.'),
                              code='banned_domain', params={'domain': domain})



from django.contrib.sites.models import Site
from .http import full_url


def email_default(request=None):
    """
    Default context for render email templates.
    """
    return {
        'SITE': Site.objects.get_current,
        'HOMEPAGE_URL': full_url,
    }



import datetime
import inspect
import os
from django.conf import settings
from django.core.management import BaseCommand
from django.utils.encoding import force_unicode
from dju_common import settings as dju_settings


class LoggingBaseCommand(BaseCommand):
    log_fn = None
    log_err_fn = None
    log_dir = dju_settings.LOG_DIR
    log_out_enabled = settings.DEBUG
    log_err_out_enabled = settings.DEBUG

    def log(self, msg, add_time=True, out=None, double_br=False, ending=None, std_stream=None, is_error=False):
        if std_stream is None:
            std_stream = self.stdout
        if out is None:
            out = self.log_out_enabled
        msg = force_unicode(msg, errors='replace')
        if double_br:
            msg += '\n\n'
        if add_time:
            msg = u'[%s] %s' % (datetime.datetime.now().strftime(u'%d.%m.%Y %H:%M:%S'), msg)
        if out:
            std_stream.write(msg, ending=ending)
        with open(os.path.join(self.log_dir, self.get_log_fn(is_error=is_error)).replace('\\', '/'), 'a') as f:
            f_ending = '\n' if ending is None else ending
            if f_ending and not msg.endswith(f_ending):
                msg += f_ending
            f.write(msg.encode('utf8'))

    def log_err(self, msg, add_time=True, out=None, double_br=False, ending=None, std_stream=None):
        if std_stream is None:
            std_stream = self.stderr
        if out is None:
            out = self.log_err_out_enabled
        self.log(msg, add_time=add_time, out=out, double_br=double_br, ending=ending, std_stream=std_stream,
                 is_error=True)

    def get_log_fn(self, is_error=False):
        get_cmd_name = lambda: os.path.splitext(os.path.basename(inspect.getfile(self.__class__)))[0]
        if is_error:
            if self.log_err_fn is None:
                return '{}.error.log'.format(get_cmd_name())
            return self.log_err_fn
        else:
            if self.log_fn is None:
                return '{}.log'.format(get_cmd_name())
            return self.log_fn



import codecs
import sys
import os
import base64
import re
import datetime
import subprocess
import traceback
from smtpd import SMTPServer
from django.core.management.base import BaseCommand
from django.utils.encoding import force_unicode, force_str
from dju_common.settings import DJU_EMAIL_DEBUG_PATH, DJU_EMAIL_DEBUG_IN_CONSOLE, DJU_EMAIL_DEBUG_IN_FILES


class DebuggingServer(SMTPServer):
    def __init__(self, *args, **kwargs):
        SMTPServer.__init__(self, *args, **kwargs)
        sys.stdout = codecs.getwriter('utf8')(sys.stdout)
        sys.stderr = codecs.getwriter('utf8')(sys.stderr)
        print('Debug email server is running. Now you can send emails to SMTP localhost:10250.')

    @staticmethod
    def _get_subject(data):
        subject_re = re.compile(ur'^Subject: (.+)$', re.IGNORECASE | re.U)
        base64_re = re.compile(ur'^=\?(.+)\?b\?(.+)\?=$', re.IGNORECASE | re.U)
        for line in data.split('\n'):
            m = subject_re.match(line)
            if m:
                subject = m.group(1).strip()
                m = base64_re.match(subject)
                if m:
                    charset, content = m.groups()
                    subject = force_unicode(base64.b64decode(content))
                return subject
        return ''

    @staticmethod
    def _get_fn(fn_base, n=None):
        if n is None:
            return os.path.join(DJU_EMAIL_DEBUG_PATH, '{}.eml'.format(fn_base)).replace('\\', '/')
        else:
            return os.path.join(DJU_EMAIL_DEBUG_PATH, '{}_{}.eml'.format(fn_base, n)).replace('\\', '/')

    def process_message(self, peer, mailfrom, rcpttos, data):
        try:
            if DJU_EMAIL_DEBUG_IN_FILES:
                if not os.path.exists(DJU_EMAIL_DEBUG_PATH):
                    os.makedirs(DJU_EMAIL_DEBUG_PATH)
                fn_base = u'{}_{}_{}_{}'.format(
                    u'_'.join(rcpttos),
                    self._get_subject(data),
                    mailfrom,
                    datetime.datetime.now().strftime(u'%Y-%m-%d_%H-%M-%S')
                )
                fn_base = re.sub(ur'[:\*\?"<>\| ]+', '_', fn_base, re.U)
                fn_base = force_str(fn_base)
                fn = self._get_fn(fn_base)
                n = 1
                while os.path.exists(fn):
                    fn = self._get_fn(fn_base, n)
                    n += 1
                f = codecs.open(fn, 'w', encoding='utf-8')
            inheaders = 1
            for line in data.split('\n'):
                if inheaders and not line:
                    if DJU_EMAIL_DEBUG_IN_FILES:
                        f.write(u'X-Peer: {}\n'.format(force_unicode(peer[0])))
                    if DJU_EMAIL_DEBUG_IN_CONSOLE:
                        print(u'X-Peer: {}'.format(force_unicode(peer[0])))
                    inheaders = 0
                line = force_unicode(line)
                if DJU_EMAIL_DEBUG_IN_FILES:
                    f.write(u'{}\n'.format(line))
                if DJU_EMAIL_DEBUG_IN_CONSOLE:
                    print(line)
        except Exception, e:
            traceback.print_exc()
            print('DebuggingServer error: {}'.format(force_unicode(e)))


class Command(BaseCommand):
    help = 'Run debug smtp server'

    def handle(self, *args, **options):
        env = os.environ.copy()
        env['PYTHONPATH'] = os.pathsep.join(sys.path)
        if os.name == 'nt':
            for k in env:
                env[k] = env[k].encode('utf-8')
        subprocess.call([
            sys.executable,
            '-m', 'smtpd', '-n', '-c', 'dju_common.management.commands.dju_debug_email_server.DebuggingServer',
            'localhost:10250'
        ], env=env)



from django import forms
from .helpers import add_css_class_to_fields_widget


BS_FORM_WIDGETS = (forms.TextInput, forms.Textarea, forms.Select, forms.FileInput)


class BootstrapForm(forms.Form):
    def __init__(self, *args, **kwargs):
        super(BootstrapForm, self).__init__(*args, **kwargs)
        add_css_class_to_fields_widget(self.fields, 'form-control')


class BootstrapModelForm(forms.ModelForm):
    def __init__(self, *args, **kwargs):
        super(BootstrapModelForm, self).__init__(*args, **kwargs)
        add_css_class_to_fields_widget(self.fields, 'form-control')
